# Copyright 2016  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# =*= License: GPL-3+ =*=


import unittest

import icklib


jessie_output = '''\
PRETTY_NAME="Debian GNU/Linux 8 (jessie)"
NAME="Debian GNU/Linux"
VERSION_ID="8"
VERSION="8 (jessie)"
ID=debian
HOME_URL="http://www.debian.org/"
SUPPORT_URL="http://www.debian.org/support"
BUG_REPORT_URL="https://bugs.debian.org/"
'''


unstable_output = '''\
PRETTY_NAME="Debian GNU/Linux stretch/sid"
NAME="Debian GNU/Linux"
ID=debian
HOME_URL="https://www.debian.org/"
SUPPORT_URL="https://www.debian.org/support"
BUG_REPORT_URL="https://bugs.debian.org/"
'''


class OsReleaseTests(unittest.TestCase):

    def test_raises_error_if_nothing_can_be_parsed(self):
        with self.assertRaises(icklib.NoOsReleaseInformation):
            icklib.parse_os_release('')

    def test_parse_jessie_os_release_ok(self):
        o = icklib.parse_os_release(jessie_output)
        self.assertEqual(o.debian_codename, 'jessie')
        self.assertEqual(o.debian_release, '8')

    def test_parse_unstable_os_release_ok(self):
        o = icklib.parse_os_release(unstable_output)
        self.assertEqual(o.pretty_codename, 'sid')
        self.assertEqual(o.debian_release, None)
