# Copyright 2016  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# =*= License: GPL-3+ =*=


import icklib


class CreateCITarball(icklib.BuildStep):

    def build(self):
        if not self.run_state.build_using_debian_ci:
            return

        self.run_state.progress['step'] = 'Creating CI tarball'
        self.run_state.logger.important('Creating CI tarball')
        with self.run_state.logger:
            tarball_target = self.targets[0]
            remote_tmp = tarball_target.mkdtemp()

            remote_git_dir = tarball_target.sync_to_target(
                self.run_state.clones[0].dirname, remote_tmp, 'git')
            remote_artifacts_dir = tarball_target.sync_to_target(
                self.run_state.build_info.create_artifacts_directory(),
                remote_tmp, 'artifacts')

            cleanly_path = self.project.copy_cleanly_to_target(
                tarball_target, remote_tmp)
            self.project.run_cleanly(
                tarball_target, remote_git_dir, cleanly_path,
                ['--results', remote_artifacts_dir,
                 '--ci',
                 '--buildno', str(self.run_state.build_info.build_number),
                 '--debian-release', tarball_target.debian_release,
                 '--debian-codename', tarball_target.debian_codename,
                 'tarball'])

            tarball_target.sync_from_target(
                remote_artifacts_dir,
                self.run_state.build_info.create_artifacts_directory())

            tarball_target.remove_all([remote_tmp])


class CreateReleaseTarballs(icklib.BuildStep):

    def build(self):
        target = self.targets[0]
        for tag in sorted(self.run_state.build_tags_using_debian_release):
            self.run_state.progress['step'] = (
                'Create release tarball for {}'.format(tag))
            self.run_state.logger.important(
                'Creating release tarball for tag {tag}',
                tag=tag)
            with self.run_state.logger:
                remote_tmp = target.mkdtemp()
                self.run_state.clones[0].checkout(tag)
                remote_git_dir = target.sync_to_target(
                    self.run_state.clones[0].dirname, remote_tmp, 'git')
                remote_artifacts_dir = target.sync_to_target(
                    self.run_state.build_info.create_artifacts_directory(),
                    remote_tmp, 'artifacts')

                cleanly_path = self.project.copy_cleanly_to_target(
                    target, remote_tmp)
                self.project.run_cleanly(
                    target, remote_git_dir, cleanly_path,
                    ['--results', remote_artifacts_dir,
                     '--release',
                     '--buildno', str(self.run_state.build_info.build_number),
                     '--debian-release', target.debian_release,
                     '--debian-codename', target.debian_codename,
                     'tarball'])

                target.sync_from_target(
                    remote_artifacts_dir,
                    self.run_state.build_info.create_artifacts_directory())

                target.remove_all([remote_tmp])
