# Copyright 2016  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# =*= License: GPL-3+ =*=


import re

import cliapp

import icklib


class FindTagsToBuildForDebianRelease(icklib.BuildStep):

    def build(self):
        tags = self._find_tags_in_repo(
            self.project.name, self.run_state.clones[0])
        seen_tags = self.run_state.project_info.seen_tags

        if seen_tags is not None:
            not_seen_tags = [x for x in tags if x not in seen_tags]
            self.run_state.build_tags_using_debian_release = not_seen_tags
            if self.run_state.build_tags_using_debian_release:
                self.run_state.logger.important(
                    'Need to build the following release tags:')
                for tag in self.run_state.build_tags_using_debian_release:
                    self.run_state.logger.important('  {tag}', tag=tag)
            else:
                self.run_state.logger.important('No new release tags')
        else:
            self.run_state.logger.important(
                'Not seen this project before, so not building release')
            self.run_state.logger.important(
                'Remembering existing release tags for future runs')
            self.run_state.build_tags_using_debian_release = []

        self.run_state.collect_debian_info = (
            self.run_state.build_tags_using_debian_release != [])
        self.run_state.project_info.seen_tags = tags

    def _find_tags_in_repo(self, project_name, clone):
        output = cliapp.runcmd(['git', 'tag', '-l'], cwd=clone.dirname)
        return [
            tag for tag in output.splitlines()
            if self._is_release_tag(project_name, tag)]

    def _is_release_tag(self, project_name, tag):
        prefix = project_name + '-'
        if not tag.startswith(prefix):
            return False
        pattern = r'^\d+\.\d+(\.\d+)*$'
        return re.match(pattern, tag[len(prefix):])
