# Copyright 2016  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# =*= License: GPL-3+ =*=


import os

import icklib


class CloneGits(icklib.BuildStep):

    def build(self):
        top_git_dir = self.statedir.get_git_directory(self.project.name)
        clones = []
        for git_spec in self.project.git_specs:
            dirname = os.path.normpath(
                os.path.join(top_git_dir, git_spec.subdir))
            self.run_state.progress['step'] = (
                'Cloning/updating git {}'.format(git_spec.url))
            self.run_state.logger.important(
                'git cloning {url} to {dir}',
                url=git_spec.url, dir=dirname)

            clone = self._clone_one(dirname, git_spec)
            clone.set_subdir(git_spec.subdir)
            clones.append(clone)

        self.run_state.clones = clones

    def _clone_one(self, dirname, git_spec):
        clone = icklib.GitClone()
        clone.set_dirname(dirname)
        clone.set_url(git_spec.url)
        clone.sync_from_url()
        clone.checkout(git_spec.branch)
        return clone
